// Fichier: ServeurSomme.cpp
#include "../Commun/Somme_h.h" // dclaration de ISomme
#include "../Commun/UUID_Fabriques.h" // UUID de FabriqueSomme
#include "Registry.h" // RegisterServer(), UnregisterServer()
#include <string>
long g_CptVerrou = 0L;

// ----------------------------------------------------
// Class COMSomme : implmentation encapsule de ISomme
// ----------------------------------------------------
class COMSomme : public ISomme
{
public:
   // Pour IUnknown
   unsigned long __stdcall AddRef ();
   unsigned long __stdcall Release ();
   HRESULT __stdcall QueryInterface (REFIID riid, void **ppv);
   // Pour ISomme
   HRESULT __stdcall Somme (int x, int y, int *Resultat);
   // constructeur et destructeur
   COMSomme ();
   ~COMSomme ();
private:
   // Compteur de rfrences (interne  l'objet)
   unsigned long m_CptRef;
};

// Pour IUnknown
unsigned long __stdcall COMSomme::AddRef ()
{
   return ++m_CptRef;
}

unsigned long __stdcall COMSomme::Release ()
{
   if (--m_CptRef != 0)
      return m_CptRef;
   delete this;
   return 0L;
}

HRESULT __stdcall COMSomme::QueryInterface (REFIID riid, void **ppv)
{
   if (riid == IID_IUnknown)
   {
      *ppv = static_cast<IUnknown *> (this);
   }
   else if (riid == IID_ISomme)
   {
      *ppv = static_cast<ISomme *> (this);
   }
   else
   {
      *ppv = 0;
      return E_NOINTERFACE;
   }
   AddRef ();
   return S_OK;
}

// Pour ISomme
HRESULT __stdcall COMSomme::Somme (int x, int y, int *Resultat)
{
   *Resultat = x + y;
   return S_OK;
}


// constructeur et destructeur
COMSomme::COMSomme () : ISomme (), m_CptRef (1)
{
   ::g_CptVerrou++;
}

COMSomme::~COMSomme ()
{
   ::g_CptVerrou--;
}

// ----------------------------------------------------
// Classe FabriqueSomme. Instancie COMSomme sur demande
// ----------------------------------------------------
class FabriqueSomme : public IClassFactory
{
public:
   // Pour IUnknown
   unsigned long __stdcall AddRef ();
   unsigned long __stdcall Release ();
   HRESULT __stdcall QueryInterface (REFIID riid, void **ppv);
   // Pour IClassFactory
   HRESULT __stdcall CreateInstance (IUnknown *pUnkOuter,
                                     REFIID riid,
                                     void **ppv);
   HRESULT __stdcall LockServer (BOOL Verrou);
   // constructeur et destructeur
   FabriqueSomme ();
   ~FabriqueSomme ();
private:
   // Compteur de rfrences (interne  l'objet)
   unsigned long m_CptRef;
};

// constructeur et destructeur
FabriqueSomme::FabriqueSomme () : IClassFactory(), m_CptRef (1)
{
}

FabriqueSomme::~FabriqueSomme ()
{
}

// Pour IUnknown
unsigned long __stdcall FabriqueSomme::AddRef ()
{
   return ++m_CptRef;
}

unsigned long __stdcall FabriqueSomme::Release ()
{
   if (--m_CptRef != 0)
      return m_CptRef;
   delete this;
   return 0L;
}

HRESULT FabriqueSomme::QueryInterface (REFIID riid, void **ppv)
{
   if (riid == IID_IUnknown)
   {
      *ppv = static_cast<IUnknown*> (this);
   }
   else if (riid == IID_IClassFactory)
   {
      *ppv = static_cast<IClassFactory*> (this);
   }
   else
   {
      *ppv = 0;
      return E_NOINTERFACE;
   }
   AddRef ();
   return S_OK;
}

// Pour IClassFactory
HRESULT FabriqueSomme::CreateInstance (IUnknown *pUnkOuter,
                                       REFIID riid,
                                       void **ppv)
{
   if (pUnkOuter != 0) // rejet de tout pUnkOuter non nul
   {
      return CLASS_E_NOAGGREGATION;
   }
   // Instanciation du composant
   COMSomme *pCOMSomme = new COMSomme;
   if (pCOMSomme == 0) // mmoire insuffisante?
      return E_OUTOFMEMORY;
   // Interrogation de l'identiti du composant
   HRESULT hr = pCOMSomme ->QueryInterface (riid, ppv);
   pCOMSomme->Release ();
   return hr;
}

HRESULT FabriqueSomme::LockServer (BOOL Verrou)
{
   if (Verrou)
      ::g_CptVerrou++;
   else
      ::g_CptVerrou--;
   return S_OK;
}

// Utilitaires de support au contexte
const std::string
   SRV_NOMFICH = "ServeurSomme.dll"
,  SRV_DESCR   = "Composant interne dmonstrateur"
,  SRV_NOM     = "Composant.Somme.InProc"
,  SRV_VERSION = "Composant.Somme.InProc.1"
;

HRESULT __stdcall DllRegisterServer ()
{
   return RegisterServer (SRV_NOMFICH,
                          CLSID_FabriqueSomme,
                          SRV_DESCR,
                          SRV_NOM,
                          SRV_VERSION,
                          0 );
}
HRESULT __stdcall DllUnregisterServer ()
{
   return UnregisterServer (CLSID_FabriqueSomme,
                            SRV_NOM,
                            SRV_VERSION);
}

HRESULT __stdcall DllGetClassObject (REFCLSID clsid, REFIID riid, void **ppv)
{
   if (clsid != CLSID_FabriqueSomme)
   {
      return CLASS_E_CLASSNOTAVAILABLE;
   }
   // Instanciation de la fabrique
   FabriqueSomme *pFab = new FabriqueSomme;
   if (pFab == 0)
      return E_OUTOFMEMORY;
   // riid est probablement IID_IClassFactory
   HRESULT hr = pFab->QueryInterface (riid, ppv);
   pFab->Release ();
   return hr;
}

HRESULT __stdcall DllCanUnloadNow ()
{
   return ((::g_CptVerrou == 0)? S_OK : S_FALSE);
}
